package com.jakubu9333.grammarManager.dao;

import org.basex.api.client.ClientSession;
import org.basex.api.client.Query;
import org.basex.core.Command;
import org.basex.core.cmd.Add;
import org.basex.core.cmd.Check;
import org.basex.core.cmd.Close;
import org.basex.core.cmd.CreateBackup;
import org.basex.core.cmd.Delete;
import org.basex.core.cmd.XQuery;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Profile;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Repository;
import org.springframework.web.server.ResponseStatusException;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.UUID;

/**
 * @author Jakub Uhlarik
 */

@Repository
@Profile("!test")
public class XmlDataRepo {

    private final ClientSession session;
    private final String dbName;

    public static final String VALIDITY_SCHEMA_NAME="grammar_check.xsd";

    public static String xQueryRuleByLeft() {
        return "descendant::*:grammar/*:rule[(@id =$param1)]";
    }

    private static String xQueryGrammarByRoot() {
        return "descendant::*:grammar[(@root = $param1)]";
    }



    public XmlDataRepo(ClientSession session,String dbName) throws IOException {
        this.dbName =dbName;
        this.session = session;

        session.execute(new Check(dbName));
        System.out.println("\nFinding schema in "+ dbName +"\n\n");
        String result = session.execute(new XQuery("db:exists(\""+dbName+"\",\""+VALIDITY_SCHEMA_NAME+"\")"));
        if (result.equals("false")) {
            String xmlSchema = Files.readString(Path.of("./xsd_schema/"+VALIDITY_SCHEMA_NAME+""));
            System.out.println("\n\nSchema not found\n");

            session.execute(new Add(VALIDITY_SCHEMA_NAME, xmlSchema));
            System.out.println("\nLoading schema\n\n");
        }

    }

    @Autowired
    public XmlDataRepo(ClientSession session) throws IOException {
        this(session,"grammarsDB");
    }

    private String executeCommand(Command command) {
        try {
            return session.execute(command);
        } catch (IOException e) {
            if (e.toString().contains("lock file")) {
                throw new ResponseStatusException(HttpStatus.INTERNAL_SERVER_ERROR, "error no concurent file add\n close database on different app\n ");
            }
            throw new ResponseStatusException(HttpStatus.BAD_REQUEST, e.toString());
        }
    }

    private String executeParamXQuery(String XQuery, String param) {
        try {
            //xquery injections
            Query cq = session.query("declare variable $param1 external; " + XQuery);
            cq.bind("param1", param, "");
            return cq.execute();
        } catch (IOException e) {
            if (e.toString().contains("lock file")) {
                throw new ResponseStatusException(HttpStatus.INTERNAL_SERVER_ERROR, "error no concurent file add\n close database on different app\n ");
            }
            throw new ResponseStatusException(HttpStatus.BAD_REQUEST, e.toString());
        }
    }

    public String getByUid(String uid) {
        return executeParamXQuery("db:open('" + dbName + "',$param1)", uid);
    }

    public String getByRoot(String root) {
        return executeParamXQuery(xQueryGrammarByRoot(), root);
    }

    public String getRuleByLeft(String id) {
        return executeParamXQuery(xQueryRuleByLeft(), id);
    }

    public String createBackup() {
        return executeCommand(new CreateBackup(dbName));
    }

    private void  validateXml(String xml) {

        xml = xml.trim();
        //prefix removal for validation with xquery
        String[] array = xml.split("<\\?xml .*\\?>");
        String xmlToValid;
        if (array.length == 1) {
            xmlToValid = array[0];
        } else {
            xmlToValid = array[1];
        }

        executeCommand(new XQuery("validate:xsd(" + xmlToValid + ",db:open(\""+dbName+"\",\"" + VALIDITY_SCHEMA_NAME + "\"))"));
    }

    public String add(String xml) {
        validateXml(xml);
        UUID uuid = UUID.randomUUID();
        executeCommand(new Add(uuid + ".grammar.xml", xml));
        return uuid + ".grammar.xml";
    }

    public String allUid() {
        return executeCommand(new XQuery("array{db:list(\"" + dbName + "\")}"));
    }

    public String delete(String uid) {
        return executeCommand(new Delete(uid));
    }

    public String getByJsgfName(String name) {
        //ignore two grammars with same name
        return executeParamXQuery("head(descendant::*:grammar/*:meta[@name='Source'][@content=$param1.jsgf']/..)", name);
    }

    //test purposes only
    public void closeServer(){
        executeCommand(new Close());
    }


    public void openAgain(String grammarDatabaseName) {
        executeCommand(new Check(grammarDatabaseName));
    }
}
