package com.jakubu9333.grammarManager.unitTest;

import com.jakubu9333.grammarManager.service.FileService;
import edu.cmu.sphinx.jsgf.JSGFGrammarException;
import edu.cmu.sphinx.jsgf.JSGFGrammarParseException;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.mockito.MockitoAnnotations;


import java.io.IOException;
import java.net.URISyntaxException;
import java.util.List;

import static org.assertj.core.api.Assertions.assertThat;


/**
 * @author Jakub Uhlarik
 */
public class FileServiceTest {

    private FileService fileService;

    @BeforeEach
    public void setUp() {
        MockitoAnnotations.openMocks(this);
        fileService = new FileService();
    }

    public static final String beginWithoutProlog = """
            <grammar version="1.0"
            xml:lang='cz'
            xmlns="http://www.w3.org/2001/06/grammar">
            <meta name="Source" content="j.jsgf"/>
            """;

    public static final String xmlBegin = """
            <?xml version="1.0"?>
            """+beginWithoutProlog;
    private final String xmlEnd="</grammar>";


    @Test
    public void xmlFormatUnknown() {
        String file1 = """
                <?xml version="1.0" ?><grammar></grammar>
                """;
        String file2 = """
                                
                                
                                
                                
                                
                                
                <?xml version="1.0" encoding="UTF-8"?><grammar></grammar>
                """;
        String file3 = """
                                
                                
                                
                                
                                
                                
                                    
                                    <?xml version="1.0" encoding="UTF-8"?><grammar></grammar>
                """;
        String file4 = """
                <?xml version="1.0" encoding="UTF-8" standalone='yes'?><grammar></grammar>
                    """;

        List<String> list = List.of(file1, file2, file3, file4);
        for (String s : list) {

            String result = fileService.getParsedFile(s, null);
            assertThat(result).isEqualTo(s);

        }


    }

    @Test
    public void jsgfFormatUnknown() {
        String file1 = """
                #JSGF V1.0; grammar j;
                  """;
        String file2 = """
                 
                 
                 
                 
                 
                 
                #JSGF V1.0 ISO8859-5; 
                grammar j;
                 """;
        String file3 = """
                                
                                
                                
                                
                                
                                
                                    
                                   #JSGF V1.0 JIS ja;grammar j;
                """;

        List<String> list = List.of(file1, file2, file3);
        for (String s : list) {
            String result = fileService.getParsedFile(s, null);
            assertThat(result).isEqualTo(xmlBegin + "</grammar>");

        }


    }

    @Test
    public void testJsgfSimpleExpConvert() throws JSGFGrammarParseException, IOException, URISyntaxException, JSGFGrammarException {
        String jsgf = """
                #JSGF V1.0;
                grammar j;
                public <rule1> = <gruk>;
                <rule2> = g*;
                <rule3> = f+;
                <rule4> = [s];
                <rule5> = l|h;
                <rule6> = g g g;
                               
                """;
        String xml = xmlBegin + """
                <rule id="rule5">
                <one-of><item>l</item><item>h</item></one-of>
                </rule>
                <rule id="rule4">
                <item repeat="0-1">s</item>
                </rule>
                <rule id="rule6">
                <item>g g g</item>
                </rule>
                <rule id="rule1" scope="public">
                <ruleref uri="#gruk"/>
                </rule>
                <rule id="rule3">
                <item repeat="1-">f</item>
                </rule>
                <rule id="rule2">
                <item repeat="0-">g</item>
                </rule>
                </grammar>
                                """;

        String result = fileService.jsgfToXml(jsgf);
        assertThat(result).isEqualTo(xml.trim());

    }
    @Test
    public void testWeightsToXml() throws JSGFGrammarParseException, IOException, URISyntaxException, JSGFGrammarException {
        String jsgf = """
                #JSGF V1.0;
                grammar j;
                public <rule1> = /12.0/j|/9.0/h|/10.0/z;
                  
                """;

        String xml1 = xmlBegin + """
                <rule id="rule1" scope="public">
                <one-of><item weight="12.0">j</item><item weight="9.0">h</item><item weight="10.0">z</item></one-of>
                </rule>
                """ + xmlEnd;

        String result = fileService.jsgfToXml(jsgf);
        assertThat(result).isEqualTo(xml1);

    }
    @Test
    public void testSpecialRules()throws JSGFGrammarParseException, IOException, URISyntaxException, JSGFGrammarException{
        String jsgf = """
                #JSGF V1.0;
                grammar j;
                public <rule1> = <NULL>|<VOID>;
                  
                """;

        String xml1 = xmlBegin + """
                <rule id="rule1" scope="public">
                <one-of><item><ruleref special="NULL"/></item><item><ruleref special="VOID"/></item></one-of>
                </rule>
                """ + xmlEnd;

        String result = fileService.jsgfToXml(jsgf);
        assertThat(result).isEqualTo(xml1);
    }

    @Test
    public void testImportTagsRules()throws JSGFGrammarParseException, IOException, URISyntaxException, JSGFGrammarException{
        String jsgf = """
                #JSGF V1.0;
                grammar j;
                import <com.dd.dds>;
                public <rule1> = xd {lul};
                  
                """;

        String xml1 = xmlBegin + """
                <meta name="seeAlso" content="com.dd.dds"/>
                <rule id="rule1" scope="public">
                <item>xd <tag>lul</tag></item>
                </rule>
                """ + xmlEnd;

        String result = fileService.jsgfToXml(jsgf);
        assertThat(result).isEqualTo(xml1);
    }

    @Test
    public void testJsgfHardExpConvert() throws JSGFGrammarParseException, IOException, URISyntaxException, JSGFGrammarException {
        String jsgf = """
                #JSGF V1.0;
                grammar j;
                public <rule1> = g*|p+|[l];
                  
                """;

        String xml1 = xmlBegin + """
                <rule id="rule1" scope="public">
                <one-of><item><item repeat="0-">g</item></item><item><item repeat="1-">p</item></item><item><item repeat="0-1">l</item></item></one-of>
                </rule>
                """ + xmlEnd;

        String result = fileService.jsgfToXml(jsgf);
        assertThat(result).isEqualTo(xml1);

    }

    @Test
    public void testXmltoJsgf() {
        //todo beware
        String jsgf = """
                #JSGF V1.0 cz;
                grammar j;
                
                public <rule1> = xd {luu};
                """;

        String xml1 = beginWithoutProlog + """
                <rule scope="public" id="rule1">
                xd <tag>luu</tag>
                </rule>
                """ + xmlEnd;

        String result = fileService.xmlToJSGF(xml1,true);
        assertThat(result).isEqualTo(jsgf);

    }
}
